#include "idt.h"

/* How every interrupt gate is defined */
typedef struct {
    unsigned short low_address; /* Lower 16 bits of handler function address */
    unsigned short sel; /* GDT Code segment selector */
    unsigned char always0;
    unsigned char flags; 
    unsigned short high_address; /* Higher 16 bits of handler function address */
} __attribute__((packed)) idt_gate;

/* A pointer to the array of interrupt handlers.
 * Assembly instruction 'lidt' will read it */
typedef struct {
    unsigned short limit;
    unsigned int base;
} __attribute__((packed)) idt_register;


#define IDT_ENTRIES 256

idt_gate idt[IDT_ENTRIES];// Array of gates
idt_register idt_reg;


void set_idt_gate(int n, unsigned int isr_address) {
    idt[n].low_address = (isr_address & 0xFFFF);
    idt[n].sel = GDT_CS;
    idt[n].always0 = 0;
    idt[n].flags = 0x8E;
    idt[n].high_address = (isr_address >> 16) & 0xFFFF;
}

void set_idt() {
    idt_reg.base = (unsigned int) &idt;
    idt_reg.limit = IDT_ENTRIES * sizeof(idt_gate) - 1;
    /* Don't make the mistake of loading &idt -- always load &idt_reg */
    __asm__ __volatile__("lidtl (%0)" : : "r" (&idt_reg));
}
