#include "irqs.h"
#include "../drivers/ports.h"

void remap_irqs() {
    port_byte_out(0x20, 0x11); // Reprogramming command PIC-M
    port_byte_out(0xA0, 0x11); // Reprogramming command PIC-S

    port_byte_out(0x21, 0x20); // vector offset PIC-M to 0x20 (32 decimal)
    port_byte_out(0xA1, 0x28); // vector offset PIC-S to 0x28 (40 decimal)

    port_byte_out(0x21, 0x04); // PIC-S at IRQ2 (0000 0100)
    port_byte_out(0xA1, 0x02); // PIC-M at IRQ9 (0000 0010)

    port_byte_out(0x21, 0x01); // 8086 CPU PIC-M
    port_byte_out(0xA1, 0x01); // 8086 CPU PIC-S

    port_byte_out(0x21, 0xFF); // disable all IRQs on PIC-M
    port_byte_out(0xA1, 0xFF); // disable all IRQs on PIC-S
    
    //port_byte_out(0x21, 0x00); // enable all IRQs on PIC-M
    //port_byte_out(0xA1, 0x00); // enable all IRQs on PIC-S
}

void mask_irq(unsigned char IRQline) {
    unsigned short port;
    unsigned char value;
    
    if(IRQline < 8) {
        port = 0x21;
    } else {
        port = 0xA1;
        IRQline -= 8;
    }
    value = port_byte_in(port) | (1 << IRQline);
    port_byte_out(port, value);     
}

void unmask_irq(unsigned char IRQline) {
    unsigned short port;
    unsigned char value;
 
    if(IRQline < 8) {
        port = 0x21;
    } else {
        port = 0xA1;
        IRQline -= 8;
    }
    value = port_byte_in(port) & ~(1 << IRQline);
    port_byte_out(port, value); 
}
