#include "isrs.h"
#include "idt.h"
#include "../drivers/screen.h"
#include "../drivers/ports.h"

// IDT vectors
void isr_install() {
    set_idt_gate(0, (unsigned int)isr0);
    set_idt_gate(1, (unsigned int)isr1);
    set_idt_gate(2, (unsigned int)isr2);
    set_idt_gate(3, (unsigned int)isr3);
    set_idt_gate(4, (unsigned int)isr4);
    set_idt_gate(5, (unsigned int)isr5);
    set_idt_gate(6, (unsigned int)isr6);
    set_idt_gate(7, (unsigned int)isr7);
    set_idt_gate(8, (unsigned int)isr8);
    set_idt_gate(9, (unsigned int)isr9);
    set_idt_gate(10, (unsigned int)isr10);
    set_idt_gate(11, (unsigned int)isr11);
    set_idt_gate(12, (unsigned int)isr12);
    set_idt_gate(13, (unsigned int)isr13);
    set_idt_gate(14, (unsigned int)isr14);
    set_idt_gate(15, (unsigned int)isr15);
    set_idt_gate(16, (unsigned int)isr16);
    set_idt_gate(17, (unsigned int)isr17);
    set_idt_gate(18, (unsigned int)isr18);
    set_idt_gate(19, (unsigned int)isr19);
    set_idt_gate(20, (unsigned int)isr20);
    set_idt_gate(21, (unsigned int)isr21);
    set_idt_gate(22, (unsigned int)isr22);
    set_idt_gate(23, (unsigned int)isr23);
    set_idt_gate(24, (unsigned int)isr24);
    set_idt_gate(25, (unsigned int)isr25);
    set_idt_gate(26, (unsigned int)isr26);
    set_idt_gate(27, (unsigned int)isr27);
    set_idt_gate(28, (unsigned int)isr28);
    set_idt_gate(29, (unsigned int)isr29);
    set_idt_gate(30, (unsigned int)isr30);
    set_idt_gate(31, (unsigned int)isr31);

    // IRQs
    // PIC-M
    set_idt_gate(32, (unsigned int)irq0);
    set_idt_gate(33, (unsigned int)irq1);
    set_idt_gate(34, (unsigned int)irq2);
    set_idt_gate(35, (unsigned int)irq3);
    set_idt_gate(36, (unsigned int)irq4);
    set_idt_gate(37, (unsigned int)irq5);
    set_idt_gate(38, (unsigned int)irq6);
    set_idt_gate(39, (unsigned int)irq7);
    // PIC-S
    set_idt_gate(40, (unsigned int)irq8);
    set_idt_gate(41, (unsigned int)irq9);
    set_idt_gate(42, (unsigned int)irq10);
    set_idt_gate(43, (unsigned int)irq11);
    set_idt_gate(44, (unsigned int)irq12);
    set_idt_gate(45, (unsigned int)irq13);
    set_idt_gate(46, (unsigned int)irq14);
    set_idt_gate(47, (unsigned int)irq15);

    set_idt(); // Load IDT descriptor in idtr register
}

void enable_interrupts(){
    __asm__ ("sti");
}

void disable_interrupts(){
    __asm__ ("cli");
}


/* To print the message which defines every exception */
char *exception_messages[] = {
    "Division By Zero\n",
    "Debug\n",
    "Non Maskable Interrupt\n",
    "Breakpoint\n",
    "Into Detected Overflow\n",
    "Out of Bounds\n",
    "Invalid Opcode\n",
    "No Coprocessor\n",

    "Double Fault\n",
    "Coprocessor Segment Overrun\n",
    "Bad TSS\n",
    "Segment Not Present\n",
    "Stack Fault\n",
    "General Protection Fault\n",
    "Page Fault\n",
    "Unknown Interrupt\n",

    "Coprocessor Fault\n",
    "Alignment Check\n",
    "Machine Check\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n",

    "Reserved\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n",
    "Reserved\n"
};

/* To print the message which defines every IRQ */
char *irq_messages[] = {
    // PIC-M
    "Programmable interval timer IRQ\n",
    "Keyboard IRQ\n",
    "8259A slave controller IRQ\n",
    "COM2 / COM4 IRQ\n",
    "COM1 / COM3 IRQ\n",
    "LPT2 IRQ\n",
    "Floppy controller IRQ\n",
    "LPT1 IRQ\n",
    // PIC-S
    "RTC IRQ\n",
    "Unassigned IRQ\n",
    "Unassigned IRQ\n",
    "Unassigned IRQ\n",
    "Mouse controller IRQ\n",
    "Math coprocessor IRQ\n",
    "Hard disk controller 1 IRQ\n",
    "Hard disk controller 2 IRQ\n"
};

void isr_handler(interrupt_stack_data interrupt_stack_data) {
        print_string("Exception interrupt detected\n", -1, -1);
        if ( interrupt_stack_data.int_no >= 0 && interrupt_stack_data.int_no <= 31){
            print_string(exception_messages[interrupt_stack_data.int_no], -1, -1);
            print_string("--------------------------------\n", -1, -1);
        } else {
            print_string("++ ERROR: Unknown exception interrupt triggered", -1, -1);
        }
}

void irq_handler(interrupt_stack_data interrupt_stack_data) {
    print_string("Inside irq_handler\n", -1, -1);
    // If IRQ was generated by PIC-M send EOI to Master, if IRQ was generated by PIC-S send EOI to Slave/Master
    if ( interrupt_stack_data.int_no >= 32 && interrupt_stack_data.int_no <= 39){
        /*if ( interrupt_stack_data.int_no == 33 ){
            print_string("Keyboard Key Pressed\n", -1, -1);
        }*/
        print_string("PIC-M IRQ detected\n", -1, -1);
        print_string("Sending EOI to PIC-M\n", -1, -1);
        port_byte_out(0x20, 0x20);
    } else if ( interrupt_stack_data.int_no >= 40 && interrupt_stack_data.int_no <= 47){
        print_string("PIC-S IRQ detected\n", -1, -1);
        print_string("Sending EOI to PIC-S\n", -1, -1);
        port_byte_out(0xA0, 0x20);
        print_string("Sending EOI to PIC-M\n", -1, -1);
        port_byte_out(0x20, 0x20);
    } else {
            print_string("++ ERROR: Unknown IRQ interrupt triggered", -1, -1);
    }
    port_byte_in(0x60); // Read keyboard key only to be able to continue testing it
    print_string("--------------------------------\n", -1, -1);
}

